@@ -17,8 +17,11 @@ from sapiopycommons.general.exceptions import SapioException
 # and one FormEntryDialogRequest methods.)
 # CR-46332 - For any functions that use temporary data types, set the data type name, display name, and plural display
 # name in the form builder.
+# FR-46716 - Add comments noting that this class is deprecated in favor of CallbackUtil.
 class PopupUtil:
     """
+    DEPRECATED: Make use of CallbackUtil as of 24.5.
+
     Methods for creating boilerplate SapioWebhookResults with client callback requests to create popup dialogs.
     """
     @staticmethod
@@ -27,6 +30,8 @@ class PopupUtil:
                    *, display_name: str | None = None, plural_display_name: str | None = None,
                    request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create a basic form entry dialog.
 
         The calling webhook must catch the FormEntryDialogResult that the client will send back.
@@ -72,12 +77,14 @@ class PopupUtil:
                           column_positions: dict[str, tuple[int, int]] = None, editable: bool | None = True,
                           *, request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create a basic form dialog for a record with displayed fields from the record data type.
 
         Makes webservice calls to get the data field and type definitions of the given data type.
         The calling webhook must catch the FormEntryDialogResult that the client will send back.
 
-        :param context: The current webhook context
+        :param context: The current webhook context.
         :param title: The title of the dialog.
         :param msg: The message to display in the dialog.
         :param fields: The data field names of the fields from the record to display in the form. Fields will be
@@ -126,6 +133,8 @@ class PopupUtil:
                            *, display_name: str | None = None, plural_display_name: str | None = None,
                            request_context: str | None = None, **kwargs) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create a form dialog with a single string field. May take additional parameters to be passed to the string field
         definition.
 
@@ -159,6 +168,8 @@ class PopupUtil:
                             *, display_name: str | None = None, plural_display_name: str | None = None,
                             request_context: str | None = None, **kwargs) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create a form dialog with a single integer field. May take additional parameters to be passed to the integer
         field definition.
 
@@ -195,6 +206,8 @@ class PopupUtil:
                            plural_display_name: str | None = None, request_context: str | None = None, **kwargs) \
             -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create a form dialog with a single double field. May take additional parameters to be passed to the double
         field definition.
 
@@ -228,6 +241,8 @@ class PopupUtil:
                     *, data_type: str = "Default", display_name: str | None = None,
                     plural_display_name: str | None = None, request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create a basic table entry dialog.
 
         The calling webhook must catch the TableEntryDialogResult that the client will send back.
@@ -262,6 +277,8 @@ class PopupUtil:
                            title: str, msg: str, fields: list[str], records: list[SapioRecord],
                            editable: bool | None = True, *, request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create a table dialog for a list of record where the columns are specific fields from the record data type.
 
         Makes webservice calls to get the data field and type definitions of the given data type.
@@ -313,6 +330,8 @@ class PopupUtil:
                                msg: str, fields: list[str], records: list[SapioRecord], multi_select: bool = True,
                                *, request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create a record selection dialog for a list of record where the columns are specific fields from the record data
         type.
 
@@ -360,6 +379,8 @@ class PopupUtil:
     def list_popup(title: str, options: list[str], multi_select: bool = False,
                    *, request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create a list dialog with the given options for the user to choose from.
 
         The calling webhook must catch the ListDialogResult that the client will send back.
@@ -378,6 +399,8 @@ class PopupUtil:
     def option_popup(title: str, msg: str, options: list[str], default_option: int = 0, user_can_cancel: bool = False,
                      *, request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create an option dialog with the given options for the user to choose from.
 
         The calling webhook must catch the OptionDialogResult that the client will send back.
@@ -400,6 +423,8 @@ class PopupUtil:
     def ok_popup(title: str, msg: str, user_can_cancel: bool = False, *, request_context: str | None = None) \
             -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create an option dialog where the only option is "OK".
 
         The calling webhook must catch the OptionDialogResult that the client will send back.
@@ -418,6 +443,8 @@ class PopupUtil:
     def yes_no_popup(title: str, msg: str, default_yes: bool = True, user_can_cancel: bool = False,
                      *, request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Create an option dialog where the only options are "Yes" and "No".
 
         The calling webhook must catch the OptionDialogResult that the client will send back.
@@ -439,6 +466,8 @@ class PopupUtil:
     def display_form_popup(title: str, field_name: str, msg: str, data_type: str = "Popup",
                            request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Deprecated for PopupUtil.text_field_popup.
         """
         return PopupUtil.string_field_popup(title, "", field_name, msg, len(msg), False, data_type,
@@ -448,6 +477,8 @@ class PopupUtil:
     def display_option_popup(title: str, msg: str, options: list[str], user_can_cancel: bool = False,
                              request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Deprecated for PopupUtil.option_popup.
         """
         return PopupUtil.option_popup(title, msg, options, 0, user_can_cancel, request_context=request_context)
@@ -455,6 +486,8 @@ class PopupUtil:
     @staticmethod
     def display_ok_popup(title: str, msg: str, request_context: str | None = None) -> SapioWebhookResult:
         """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
         Deprecated for PopupUtil.ok_popup.
         """
         return PopupUtil.ok_popup(title, msg, False, request_context=request_context)
